@@ -41,28 +41,33 @@ sub run {
 
 sub do_update {
     my ($self, $cotarget, $update_target) = @_;
-    my $xdroot = $cotarget->root ($self->{xd});
+    my ($xdroot, $newroot) = map { $_->root ($self->{xd}) } ($cotarget, $update_target);
     # unanchorified
     my ($path, $copath) = @{$cotarget}{qw/path copath/};
     my $report = $update_target->{report};
+    my $kind = $newroot->check_path ($update_target->{path});
+    die loc("path %1 does not exist.\n", $update_target->{path})
+	if $kind == $SVN::Node::none;
 
     print loc("Syncing %1(%2) in %3 to %4.\n", @{$cotarget}{qw( depotpath path copath )},
 	      $update_target->{revision});
-    unless ($xdroot->check_path ($cotarget->{path}) == $SVN::Node::dir) {
+    if ($kind == $SVN::Node::file ) {
 	$cotarget->anchorify;
 	$update_target->anchorify;
+	# can't use $cotarget->{path} directly since the (rev0, /) hack
+	($path, $copath) = @{$cotarget}{qw/path copath/};
     }
-    else {
-	mkdir ($cotarget->{copath})
-	    unless $self->{check_only};
-    }
+    $cotarget = $cotarget->new (path => '/')
+	if $xdroot->check_path ($cotarget->path) == $SVN::Node::none;
+    mkdir ($cotarget->{copath}) or die $!
+	unless $self->{check_only} || -e $cotarget->{copath};
 
     my $merge = SVK::Merge->new
 	(repos => $cotarget->{repos}, base => $cotarget, base_root => $xdroot,
-	 no_recurse => $self->{nonrecursive}, report => $report,
+	 no_recurse => $self->{nonrecursive}, report => $report, nodelay => 1,
 	 src => $update_target, xd => $self->{xd}, check_only => $self->{check_only});
     $merge->run ($self->{xd}->get_editor (copath => $copath, path => $path,
-					  oldroot => $xdroot, newroot => $update_target->root,
+					  oldroot => $xdroot, newroot => $newroot,
 					  revision => $update_target->{revision},
 					  anchor => $cotarget->{path},
 					  target => $cotarget->{targets}[0] || '',
