@@ -11,6 +11,7 @@ use File::Path;
 use SVK;
 use SVK::XD;
 use strict;
+use Carp;
 
 our @TOCLEAN;
 END {
@@ -85,21 +86,21 @@ sub cleanup_test {
 
 sub append_file {
     my ($file, $content) = @_;
-    open my ($fh), '>>', $file or die "can't append $file: $!";
+    open my ($fh), '>>:raw', $file or die "can't append $file: $!";
     print $fh $content;
     close $fh;
 }
 
 sub overwrite_file {
     my ($file, $content) = @_;
-    open my ($fh), '>', $file or die $!;
+    open my ($fh), '>:raw', $file or confess $!;
     print $fh $content;
     close $fh;
 }
 
 sub is_file_content {
     my ($file, $content, $test) = @_;
-    open my ($fh), '<', $file or die $!;
+    open my ($fh), '<:raw', $file or confess $!;
     local $/;
     is (<$fh>, $content, $test);
 }
@@ -109,14 +110,12 @@ sub is_output {
     $svk->$cmd (@$arg);
     is_deeply ([split ("\n", $output)], $expected,
 	       $test || join(' ', $cmd, @$arg));
-    diag $@ if $@;
 }
 
 sub is_output_like {
     my ($svk, $cmd, $arg, $expected, $test) = @_;
     $svk->$cmd (@$arg);
     ok ($output =~ m/$expected/, $test || join(' ', $cmd, @$arg));
-    diag $@ if $@;
 }
 
 require SVN::Simple::Edit;
